"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const child_process_1 = require("child_process");
const logger = __importStar(require("./utils/logger"));
const figlet = __importStar(require("figlet"));
const clui_1 = require("clui");
const chalk = __importStar(require("chalk"));
const fs = __importStar(require("fs"));
/**
 * Builds a lambda project in specified directory with giving name.
 * @param dir user directory the project is to be built.
 * @param service name used the represent the lambda service in aws.
 * @param app project application name.
 */
function buildLambda(dir, service, app) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            const srcPackagePath = path_1.join(__dirname, './templates/configs/package.json');
            const srcServerlessPath = path_1.join(__dirname, './templates/serverless.yml');
            const destServerlessPath = path_1.join(dir, './serverless.yml');
            const destPackagePath = path_1.join(dir, './package.json');
            displayLogo();
            fs.mkdirSync(dir);
            fs.mkdirSync(path_1.join(dir, './src'));
            yield Promise.all([
                copyFile(path_1.join(__dirname, './templates/src/index.ts'), path_1.join(dir, './src/index.ts')),
                copyFolder(path_1.join(__dirname, './templates/resources'), path_1.join(dir, '/resources')),
                copyFolder(path_1.join(__dirname, './templates/configs'), path_1.resolve(dir))
            ]);
            yield Promise.all([
                writeServiceNameToFile(srcServerlessPath, destServerlessPath, service, app),
                writeServiceNameToFile(srcPackagePath, destPackagePath, service, null)
            ]);
            yield runCmdHandler(dir, 'npm --silent install', 'installing node modules... ');
            logger.info('Your lambda was successfully build.');
            resolve();
        }
        catch (error) {
            logger.error(`buildLambda`, `Error occurred while creating lambda`);
            reject(error);
        }
    }));
}
exports.buildLambda = buildLambda;
/**
 * Deploys lambda and all it's services to aws.
 * @param cmd lambda deploy command that should run on user's computer.
 */
function deployLambda(cmd) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            displayLogo();
            yield runCmdHandler(path_1.resolve(process.cwd()), cmd, 'deploying your lambda to aws... ');
            resolve();
        }
        catch (error) {
            logger.error(`deployLambda`, `Error occurred while deploying lambda`);
            reject(error);
        }
    }));
}
exports.deployLambda = deployLambda;
/**
 * Removes lambda and all it's services from aws.
 * @param cmd lambda remove command that should run on user's computer.
 */
function removeLambda(cmd) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            displayLogo();
            yield runCmdHandler(path_1.resolve(process.cwd()), cmd, 'removing your lambda from aws... ');
            resolve();
        }
        catch (error) {
            logger.error(`removeLambda`, `Error occurred while removing lambda`);
            reject(error);
        }
    }));
}
exports.removeLambda = removeLambda;
/**
 * Copies a files in folder from directory to another
 * @param src source folder that is to be copied
 * @param dest destination directory that the files is to be copied to
 */
function copyFolder(src, dest) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            const jobs = [];
            const files = fs.readdirSync(src);
            if (!fs.existsSync(dest))
                fs.mkdirSync(dest);
            for (const file of files)
                jobs.push(copyFile(path_1.join(src, file), path_1.join(dest, file)));
            yield Promise.all(jobs);
            resolve();
        }
        catch (error) {
            logger.error(`copyFolder`, `Error occurred while copying folder contents`);
            reject(error);
        }
    }));
}
/**
 * Copies a file from directory to another
 * @param src source file path that is to be copied
 * @param dest destination path that the file is to be copied to
 */
function copyFile(src, dest) {
    return new Promise((resolve, reject) => {
        fs.copyFile(src, dest, error => {
            if (error) {
                logger.error(`copyFile`, `Error occurred while copying file`);
                reject(error);
                return;
            }
            resolve();
        });
    });
}
/**
 * Reads file into memory then replaces any text that matches the regular expression
 * with the provided service name.
 * @param srcPath source path that the file will be read from.
 * @param destPath destination path that the file should be written in.
 * @param service name used the represent the lambda service in aws.
 * @param app project application name.
 */
function writeServiceNameToFile(srcPath, destPath, service, app) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            const data = fs.readFileSync(srcPath);
            let result = data.toString().replace(/{service-name}/g, service);
            if (app !== null)
                result = result.replace(/{application-name}/g, app.toLowerCase());
            fs.writeFileSync(destPath, result, 'utf8');
            resolve();
        }
        catch (error) {
            logger.error(`writeServiceNameToFile`, `Error occurred while while writing service name to file`);
            reject(error);
        }
    }));
}
/**
 * Runs the given the command in the specified user directory.
 * @param dir user directory that the command should run in.
 * @param cmd command that should run in the provided directory.
 * @param msg status message that is displayed to the user.
 */
function runCmdHandler(dir, cmd, msg) {
    return new Promise((resolve, reject) => {
        try {
            const loading = new clui_1.Spinner(msg, ['⣾', '⣽', '⣻', '⢿', '⡿', '⣟', '⣯', '⣷']);
            const process = spawnProcess(dir, cmd);
            let count = -1;
            process.stdout.setEncoding('utf8');
            loading.start();
            process.stdout.on('data', data => {
                const regex = /^[.]+$/g;
                // skips output result that only contains the character '.'
                if (!regex.test(data.trim()) && data.trim() !== '') {
                    count++;
                    if (count === 0)
                        logger.info('\n');
                    logger.debug(`Output`, data.trim());
                }
            });
            process.stderr.on('data', (data) => {
                count++;
                if (count === 0)
                    logger.info('\n');
                logger.error(`runCmdHandler`, data);
            });
            process.on('exit', code => {
                count++;
                if (count === 0)
                    logger.info('\n');
                logger.debug('finished with exist code', `${code.toString()}`);
                loading.stop();
                resolve();
            });
        }
        catch (error) {
            reject(error);
            logger.error(`runCmdHandler`, `Error occurred while running ${cmd}`);
        }
    });
}
/**
 * Checks which platform the program is running on then request to
 * run the given command on that platform.
 * @param dir user directory that the command should run in.
 * @param cmd command that should run in the provided directory.
 */
function spawnProcess(dir, cmd) {
    return (process.platform === 'win32')
        ? spawnWindowsProcess(dir, cmd)
        : spawnLinuxProcess(dir, cmd);
}
exports.spawnProcess = spawnProcess;
/**
 * Spawns a windows process to run the given command in the specified directory.
 * @param dir user directory that the command should run in.
 * @param cmd command that should run in the provided directory.
 */
function spawnWindowsProcess(dir, cmd) {
    return child_process_1.spawn("cmd.exe", ["/c", cmd], { cwd: dir });
}
/**
 * Spawns a linux process to run the given command in the specified directory.
 * @param dir user directory that the command should run in.
 * @param cmd command that should run in the provided directory.
 */
function spawnLinuxProcess(dir, cmd) {
    const parts = cmd.split(/\s+/);
    return child_process_1.spawn(parts[0], parts.slice(1), { cwd: dir });
}
/**
 * Displays the branding logo on the screen.
 */
function displayLogo() {
    console.clear();
    console.log(chalk.default.blueBright(figlet.textSync('Lambda Manager', { horizontalLayout: 'default' })));
}
